////////////////////////////////////////////////////////////////////// 
// lamp/MarkerMap.cpp 
// (c) 2000-2013 Goncalo Abecasis
// 
// This file is distributed as part of the LAMP source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile LAMP.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Thursday April 18, 2013
// 
 
#include "MarkerMap.h"
#include "StringMap.h"
#include "StringHash.h"
#include "QuickIndex.h"
#include "IntArray.h"
#include "Error.h"

#include <ctype.h>

void MarkerMap::Load(const char * filename, bool required)
   {
   FILE * f = fopen(filename, "rt");

   if (f == NULL)
      if (required)
         error("Cannot open map file [%s]", filename);
      else
         {
         chromosome = -1;
         return;
         }

   Load(f);

   fclose(f);
   }

void MarkerMap::Load(FILE * file)
   {
   String           input;
   StringArray      tokens;
   StringDoubleHash hash;

   chromosome = -1;

   while (!feof(file))
      {
      input.ReadLine(file);
      tokens.Clear();
      tokens.AddTokens(input);

      if (tokens.Length() == 0) continue;

      if (tokens.Length() != 3)
         {
         printf("Error reading map file.\n"
                "   Each line should list CHR MARKER POSITION, separated by spaces\n"
                "   But the following line does not make sense:\n\n%s\n\n",
                (const char *) input);
         continue;
         }

      if (!isdigit(tokens[0][0]) && toupper(tokens[0][0]) != 'X')
         continue;

      int this_chromosome = toupper(tokens[0][0]) == 'X' ? 23 : tokens[0].AsInteger();

      if (chromosome == -1)
         chromosome = this_chromosome;
      else if (chromosome != this_chromosome)
         error("Error reading map file.\n"
               "    All markers should be on the same chromosome.\n");

      hash.SetDouble(tokens[1], tokens[2].AsDouble() * 0.01);
      }

   IntArray slotIndex;

   markers.Dimension(hash.Entries());
   positions.Dimension(hash.Entries());
   slotIndex.Dimension(hash.Entries());

   for (int i = 0, out = 0; i < hash.Capacity(); i++)
      if (hash.SlotInUse(i))
         {
         slotIndex[out] = i;
         positions[out++] = hash.Double(i);
         }

   QuickIndex index;

   index.Index(positions);

   for (int i = 0; i < markers.Length(); i++)
      {
      positions[i] = hash.Double(slotIndex[index[i]]);
      markers[i] = hash[slotIndex[index[i]]];
      }
   }

 
